@@ -200,7 +200,7 @@ class Network:
 
 		# Generate instructions
 		code += "'''\n"
-		code += "The code bellow is automatically generated by NuPIC Studio and aims help you to quickly integrate\n"
+		code += "The code below is automatically generated by NuPIC Studio and aims to help you to quickly integrate\n"
 		code += "this HTM network into your application using the NuPIC Network API.\n"
 		code += "For use this code, just create a new Python file and copy/paste these lines into it. Run it and voila!\n"
 		code += "'''\n"
